package batch;

import java.io.IOException;
import java.util.Date;

import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;
import org.neo4j.graphdb.Transaction;
import org.neo4j.index.IndexHits;
import org.neo4j.index.lucene.LuceneIndexService;
import org.neo4j.kernel.EmbeddedGraphDatabase;

/**
 * Elimina dal grafo le categorie che non hanno valore semantico
 * e che sono riportate nel file precedentemente generato da EstrazioneCatagorieInterne
 * 
 * */
public class EliminazioneCategorieInterne {

	public static void main(String argc[]) throws IOException {
		new EliminazioneCategorieInterne();
	}


	private EmbeddedGraphDatabase graph;
	private LuceneIndexService index;


	private int manipolazioni=0;
	private Transaction tx;


	public EliminazioneCategorieInterne() throws IOException{
		System.out.println("Avvio il database e apro la transazione...");
		graph=new EmbeddedGraphDatabase( "neo4j-db/" );
		tx=graph.beginTx();
		
		System.out.println("Fatto. Avvio l'indice...");
		index = new LuceneIndexService(graph);
		System.out.println("indice avviato. Apro l'elenco delle categorie da eliminare");
		Lettore l=new Lettore("export_Wikipedia administration.txt");
		Scrittore s=new Scrittore("eliminazione_categorie_interne.log");
		try{
			
			s.scrivi("programma avviato "+new Date().toString());
			System.out.println("Fatto, inizio il lavoro");
			String letta;
			int riga=0;
			int manipolazioni=0;
			while((letta=l.leggi())!=null){
				riga++;
				manipolazioni++;
				//per ogni nodo chiudo e riapro la transazione
				//questo perchÃ© alcuni nodi hanno moltissime relazioni e darebbero problemi i "nodi fantasma"
				tx.success();
				tx.finish();
				tx=graph.beginTx();
				
				System.out.println(letta);
				int idcat=Integer.parseInt(letta.split(" ")[1]);
				IndexHits<Node> nodi = index.getNodes("idcat", idcat);
				
				for(Node n:nodi){
					System.out.println("nodi.size:"+nodi.size()+" properietÃ : "+n.getPropertyKeys());
				}
				if(nodi.size()==0){
					s.scrivi("Nessuna categoria con idcat="+idcat+" alla riga "+riga);
					continue;
				}
				
				if(nodi.size()==1){
					Node el=index.getSingleNode("idcat", idcat);
					if (el==null){
						System.out.println("ERRORE NULL");
						s.scrivi("Errore NULL");
						continue;
					}
					//prima devo rimuovere le relazioni attaccate
					while(el.hasRelationship())
					for(Relationship r:el.getRelationships(Direction.BOTH)){
					System.out.println("elimino relazione "+r.getEndNode().getProperty("name")+" - "+r.getStartNode().getProperty("name"));
						r.delete();
						manipolazioni++;
						if(manipolazioni%100==0){
							//ogni 100 righe svuoto e ricomincio la transazione
							tx.success();
							tx.finish();
							tx=graph.beginTx();
						}
						//tentativo:interrompi il ciclo, che Ã¨ fasullo:non si itera su una cosa che si modifica
						break;
						
					}
					el.delete();
					s.scrivi("**Elimino categoria con idcat="+idcat+" alla riga "+riga);
				}
				if(nodi.size()>1){
					s.scrivi("------------ERRORE");
					s.scrivi("PiÃ¹ di una categoria con idcat="+idcat+" alla riga "+riga);
					s.scrivi("---------");
				}
				

			}
			s.chiudi();
			l.chiudi();
		}
		catch(Exception e){
			System.err.println("eccezione catturata:");
			e.printStackTrace();
		}
		finally{
			tx.success();
			tx.finish();
			l.chiudi();
			s.chiudi();
			graph.shutdown();
			System.out.println("FINE, programma terminato");
		}
	}


}
